unit Options;

interface

uses
   Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
   Dialogs, StdCtrls, Buttons, ExtCtrls, TntStdCtrls, TntExtCtrls, TntWideStrings, TntClasses,
   TntButtons, TntForms, TntDialogs, TntSysUtils, ShellApi, ProcessViewer, Math,
   xmldom, XMLIntf, msxmldom, XMLDoc;

type
   TfrmOptions = class(TTNTForm)
      pnlAll: TTntPanel;
      gbGeneral: TTntGroupBox;
      cbLock: TTntCheckBox;
      gbVirtualBox: TTntGroupBox;
      gbQEMU: TTntGroupBox;
      btnOK: TTntBitBtn;
      btnCancel: TTntBitBtn;
      lblVBExePath: TTntLabel;
      edtVBExePath: TTntEdit;
      btnBrowseForVBExe: TTntSpeedButton;
      cbSecondDrive: TTntCheckBox;
      cbUseVboxmanage: TTntCheckBox;
      cbDirectly: TTntCheckBox;
      lblQExePath: TTntLabel;
      edtQExePath: TTntEdit;
      btnBrowseForQExe: TTntSpeedButton;
      edtDefaultParameters: TTntEdit;
      lblDefaultParameters: TTntLabel;
      odSearchExe: TTntOpenDialog;
      lblWaitTime: TTntLabel;
      edtWaitTime: TTntEdit;
      cbBackup: TTntCheckBox;
      cbDontWarn: TTntCheckBox;
      cbCPUVirtualizationOption: TTntCheckBox;
      gbDefaultVMType: TTntGroupBox;
      gbUpdateMethod: TTntGroupBox;
      cbRemoveDrive: TTntCheckBox;
      cbAutoDetect: TTntCheckBox;
      cbListOnlyUSBDrives: TTntCheckBox;
      btnChooseFont: TTntBitBtn;
      fdListViewFont: TFontDialog;
      cbAutomaticFont: TTntCheckBox;
      cbEscapeKeyClosesMain: TTntCheckBox;
      pnlType: TPanel;
      sbQEMU: TTntSpeedButton;
      sbVirtualBox: TTntSpeedButton;
      lblLanguage: TTntLabel;
      cmbLanguage: TTntComboBox;
      xmlTemp: TXMLDocument;
      procedure cbUseVboxmanageClick(Sender: TObject);
      procedure cbDirectlyClick(Sender: TObject);
      procedure btnBrowseForVBExeClick(Sender: TObject);
      procedure btnBrowseForQExeClick(Sender: TObject);
      procedure btnOKClick(Sender: TObject);
      procedure edtWaitTimeKeyPress(Sender: TObject; var Key: Char);
      procedure AllKeyDown(Sender: TObject; var Key: Word;
         Shift: TShiftState);
      procedure pmChangeMessageBox(var Msg: TMessage); message WM_USER + 1024;
      procedure edtDefaultParametersChange(Sender: TObject);
      procedure cbAutoDetectClick(Sender: TObject);
      procedure TntFormCreate(Sender: TObject);
      procedure btnChooseFontClick(Sender: TObject);
      procedure cbAutomaticFontClick(Sender: TObject);
      procedure sbVirtualBoxClick(Sender: TObject);
      procedure sbQEMUClick(Sender: TObject);
   private
      originaledtVBExePathWindowProc: TWndMethod;
      originaledtQExePathWindowProc: TWndMethod;
      procedure edtVBExePathWindowProc(var Msg: TMessage);
      procedure edtQExePathWindowProc(var Msg: TMessage);
      { Private declarations }
   public
      { Public declarations }
   end;

var
   frmOptions: TfrmOptions;

implementation

uses MainForm;

{$R *.dfm}

procedure TfrmOptions.pmChangeMessageBox(var Msg: TMessage);
var
   MBRect: TRect;
   i, x, y, w, h: integer;
   ws: WideString;
   p: PAnsiChar;
begin
   GetProcessList;
   i := 0;
   try
      p := MAKEINTRESOURCE(WC_DIALOG);
      ws := WideString('#' + IntToStr(Integer(p)));
   except
      ws := '#32770';
   end;
   while i <= High(ProcessInfo) do
   begin
      if ProcessInfo[i].WClass = ws then
         if ProcessInfo[i].WCaption = msgBoxCaption then
            if GetParent(ProcessInfo[i].Handle) = Handle then
               Break;
      Inc(i);
   end;
   if i <= High(ProcessInfo) then
   begin
      try
         if not GetWindowRect(ProcessInfo[i].Handle, MBRect) then
            Exit;
      except
         Exit;
      end;
      w := MBRect.Right - MBRect.Left;
      h := MBRect.Bottom - MBRect.Top;

      x := Left + ((Width - w) div 2);
      if x < 0 then
         x := 0
      else if x + w > Screen.Width then
         x := Screen.Width - w;

      y := Top + ((Height - h) div 2);
      if y < 0 then
         y := 0
      else if y + h > Screen.Height then
         y := Screen.Height - h;
      try
         SetWindowPos(ProcessInfo[i].Handle, 0, x, y, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE
            or SWP_NOZORDER);
      except
      end;
   end;
end;

procedure TfrmOptions.cbUseVboxmanageClick(Sender: TObject);
begin
   cbDirectly.Checked := not cbUseVboxmanage.Checked and (not cbAutoDetect.Checked);
   cbAutoDetect.Checked := not cbDirectly.Checked and (not cbUseVboxmanage.Checked);
end;

procedure TfrmOptions.cbDirectlyClick(Sender: TObject);
begin
   cbUseVboxmanage.Checked := not cbDirectly.Checked and (not cbAutoDetect.Checked);
   cbAutoDetect.Checked := not cbDirectly.Checked and (not cbUseVboxmanage.Checked);
end;

procedure TfrmOptions.btnBrowseForVBExeClick(Sender: TObject);
var
   FolderName: WideString;
begin
   FolderName := WideExtractFilePath(odSearchExe.FileName);
   odSearchExe.FileName := '';
   if FolderName = '' then
   begin
      if odSearchExe.InitialDir = '' then
         odSearchExe.InitialDir := GetEnvVarValue('ProgramFiles');
   end
   else
      odSearchExe.InitialDir := FolderName;
   if odSearchExe.Execute then
   begin
      edtVBExePath.Text := odSearchExe.FileName;
      edtVBExePath.SelStart := Length(edtVBExePath.Text);
      edtVBExePath.SelLength := 0;
   end;
end;

procedure TfrmOptions.btnBrowseForQExeClick(Sender: TObject);
var
   FolderName: WideString;
begin
   FolderName := WideExtractFilePath(odSearchExe.FileName);
   odSearchExe.FileName := '';
   if FolderName = '' then
   begin
      if odSearchExe.InitialDir = '' then
         odSearchExe.InitialDir := GetEnvVarValue('ProgramFiles');
   end
   else
      odSearchExe.InitialDir := FolderName;
   if odSearchExe.Execute then
   begin
      edtQExePath.Text := odSearchExe.FileName;
      edtQExePath.SelStart := Length(edtQExePath.Text);
      edtQExePath.SelLength := 0;
   end;
end;

procedure TfrmOptions.btnOKClick(Sender: TObject);
var
   i: Integer;
begin
   i := StrToIntDef(Trim(edtWaitTime.Text), -1);
   if (i < 0) or (i > 20000) then
   begin
      msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
      PostMessage(Handle, WM_USER + 1024, 0, 0);
      MessageBoxW(Handle,
         PWideChar(GetLangTextDef(idxOptions, ['Messages', 'ProperWaitTime'], 'Please set a proper value for the wait time (0..20000) !')),
         PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      edtWaitTime.SetFocus;
      Exit;
   end;
   if (Trim(edtVBExePath.Text) <> '') and (not WideFileExists(Trim(edtVBExePath.Text))) then
   begin
      msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
      PostMessage(Handle, WM_USER + 1024, 0, 0);
      MessageBoxW(Handle,
         PWideChar(GetLangTextFormatDef(idxOptions, ['Messages', 'FileDoesntExist'], ['VirtualBox', Trim(edtVBExePath.Text)], 'The file "%s" doesn''t exist !'#13#10'Please clear the %s Exe Path from the edit box if you don''t want to use it...')),
         PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      Exit;
   end;
   if (Trim(edtQExePath.Text) <> '') and (not WideFileExists(Trim(edtQExePath.Text))) then
   begin
      msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
      PostMessage(Handle, WM_USER + 1024, 0, 0);
      MessageBoxW(Handle,
         PWideChar(GetLangTextFormatDef(idxOptions, ['Messages', 'FileDoesntExist'], ['QEMU', Trim(edtQExePath.Text)], 'The file "%s" doesn''t exist !'#13#10'Please clear the %s Exe Path from the edit box if you don''t want to use it...')),
         PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      Exit;
   end;
   if cmbLanguage.ItemIndex = -1 then
   begin
      msgBoxCaption := 'Warning';
      PostMessage(Handle, WM_USER + 1024, 0, 0);
      MessageBoxW(Handle,
         PWideChar(WideString('A language for the interface must be selected...!')),
         PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      Exit;
   end;
   ModalResult := mrOk;
end;

procedure TfrmOptions.edtWaitTimeKeyPress(Sender: TObject; var Key: Char);
begin
   if not (Key in ['0'..'9', #8]) then
      Key := #0;
end;

procedure TfrmOptions.AllKeyDown(Sender: TObject; var Key: Word;
   Shift: TShiftState);
begin
   if (Shift = []) and (Key = VK_F1) then
      frmMain.OpenInternetHelp(Self.Handle, DefSiteHelp);
end;

procedure TfrmOptions.edtDefaultParametersChange(Sender: TObject);
begin
   edtDefaultParameters.ShowHint := Trim(edtDefaultParameters.Text) = WideString('-name "USB Boot Test" -boot c -m 512 -soundhw sb16');
end;

procedure TfrmOptions.cbAutoDetectClick(Sender: TObject);
begin
   cbUseVboxmanage.Checked := not cbDirectly.Checked and (not cbAutoDetect.Checked);
   cbDirectly.Checked := not cbUseVboxmanage.Checked and (not cbAutoDetect.Checked);
end;

procedure TfrmOptions.TntFormCreate(Sender: TObject);
var
   NewWidth: Integer;
   OldText: WideString;
begin
   NewWidth := Min(Max(StrToIntDef(GetLangTextDef(idxOptions, ['Width'], IntToStr(Width)), Width), 100), 1000);
   Caption := GetLangTextDef(idxOptions, ['Caption'], 'Options');
   lblWaitTime.Caption := GetLangTextDef(idxOptions, ['Labels', 'WaitTime'], 'Wait time to flush system data before dismounting (ms):');
   edtWaitTime.Width := edtWaitTime.Left + edtWaitTime.Width - lblWaitTime.Left - lblWaitTime.Width - 5 + NewWidth - Width;
   edtWaitTime.Left := lblWaitTime.Left + lblWaitTime.Width + 5 - NewWidth + Width;
   lblVBExePath.Caption := GetLangTextDef(idxOptions, ['Labels', 'ExePath'], 'Exe path:');
   edtVBExePath.Width := edtVBExePath.Left + edtVBExePath.Width - lblVBExePath.Left - lblVBExePath.Width - 5 + NewWidth - Width;
   edtVBExePath.Left := lblVBExePath.Left + lblVBExePath.Width + 5 - NewWidth + Width;
   lblQExePath.Caption := GetLangTextDef(idxOptions, ['Labels', 'ExePath'], 'Exe path:');
   edtQExePath.Width := edtQExePath.Left + edtQExePath.Width - lblQExePath.Left - lblQExePath.Width - 5 + NewWidth - Width;
   edtQExePath.Left := lblQExePath.Left + lblQExePath.Width + 5 - NewWidth + Width;
   lblDefaultParameters.Caption := GetLangTextDef(idxOptions, ['Labels', 'DefCommParam'], 'The default command line parameters:');
   odSearchExe.Title := GetLangTextDef(idxOptions, ['Dialogs', 'LoadExe', 'Title'], 'Open');
   odSearchExe.Filter := GetLangTextDef(idxOptions, ['Dialogs', 'LoadExe', 'Filter'], 'Exe files (*.exe)|*.exe|All files (*.*)|*.*');
   OldText := btnChooseFont.Caption;
   btnChooseFont.Caption := GetLangTextDef(idxOptions, ['Buttons', 'ChooseFont'], 'Choose font');
   Canvas.Font.Assign(btnChooseFont.Font);
   btnChooseFont.Left := btnChooseFont.Left + Canvas.TextWidth(OldText) - Canvas.TextWidth(btnChooseFont.Caption);
   btnChooseFont.Width := btnChooseFont.Width - Canvas.TextWidth(OldText) + Canvas.TextWidth(btnChooseFont.Caption);
   cbAutomaticFont.Width := btnChooseFont.Left - 5 - cbAutomaticFont.Left;
   btnOK.Caption := GetLangTextDef(idxOptions, ['Buttons', 'OK'], 'OK');
   btnCancel.Caption := GetLangTextDef(idxOptions, ['Buttons', 'Cancel'], 'Cancel');
   cbLock.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'LockVol'], 'Lock the volumes on the drive before dismounting (safe but sometimes slow)');
   cbSecondDrive.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'SecDrive'], 'Show a second drive option');
   cbBackup.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'BackupOldEntries'], 'Backup the old entries before saving the new ones');
   cbDontWarn.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'DontWarn'], 'Don''t warn when not able to save the configuration or the entries file (read only medium)');
   cbListOnlyUSBDrives.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'OnlyUSB'], 'List only USB drives');
   cbAutomaticFont.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'AutomaticFont'], 'Font size, style and color will be set automatically');
   cbEscapeKeyClosesMain.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'EscapeKey'], 'Escape key closes the main window');
   cbAutoDetect.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'AutodetectMethod'], 'try to autodetect the most appropriate for the given situation');
   cbUseVboxmanage.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'UseVBMethod'], 'use VBoxManage.exe command line (slower)');
   cbDirectly.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'DirectWrtMethod'], 'directly (faster, but VB Manager must be closed)');
   cbCPUVirtualizationOption.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'VirtOpt'], 'Add "Enable VT-x/AMD-V" option to the Add/Edit window');
   cbRemoveDrive.Caption := GetLangTextDef(idxOptions, ['Checkboxes', 'RemDrives'], 'Remove the drive(s) from the VM after closing');
   gbGeneral.Caption := GetLangTextDef(idxOptions, ['Groupboxes', 'General'], 'General');
   gbDefaultVMType.Caption := GetLangTextDef(idxOptions, ['Groupboxes', 'DefaultVMType'], 'Default VM type when adding a new entry');
   gbUpdateMethod.Caption := GetLangTextDef(idxOptions, ['Groupboxes', 'UpdateVMMethod'], 'Method to update the VM configuration file (*.vbox)');
   btnBrowseForVBExe.Hint := GetLangTextDef(idxOptions, ['Hints', 'BrowseForExe'], 'click to browse for exe');
   btnBrowseForQExe.Hint := GetLangTextDef(idxOptions, ['Hints', 'BrowseForExe'], 'click to browse for exe');
   edtDefaultParameters.Hint := GetLangTextDef(idxOptions, ['Hints', 'DefaultParam'], 'Basic parameters for x86/x64 version');
   cmbLanguage.Width := cmbLanguage.Left + cmbLanguage.Width - lblLanguage.Left - lblLanguage.Width - 5 + NewWidth - Width;
   cmbLanguage.Left := lblWaitTime.Left + lblLanguage.Width + 5 - NewWidth + Width;
   Width := NewWidth;
   Left := frmMain.Left + ((frmMain.Width - Width) div 2);
   if Left < Screen.WorkAreaLeft then
      Left := Screen.WorkAreaLeft
   else if Left + Width > Screen.WorkAreaRect.Right then
      Left := Screen.WorkAreaRect.Right - Width;

   Top := frmMain.Top + ((frmMain.Height - Height) div 2);
   if Top < Screen.WorkAreaTop then
      Top := Screen.WorkAreaTop
   else if Top + Height > Screen.WorkAreaRect.Bottom then
      Top := Screen.WorkAreaRect.Bottom - Height;
   originaledtVBExePathWindowProc := edtVBExePath.WindowProc;
   edtVBExePath.WindowProc := edtVBExePathWindowProc;
   originaledtQExePathWindowProc := edtQExePath.WindowProc;
   edtQExePath.WindowProc := edtQExePathWindowProc;
   DragAcceptFiles(edtVBExePath.Handle, True);
   DragAcceptFiles(edtQExePath.Handle, True);
end;

procedure TfrmOptions.btnChooseFontClick(Sender: TObject);
var
   FontName: AnsiString;
   FontSize: Word;
   FontBold, FontItalic, FontUnderline, FontStrikeOut: Boolean;
   FontColor: TColor;
   FontScript: Word;
begin
   with fdListViewFont.Font do
   begin
      FontName := Name;
      FontSize := Size;
      FontBold := fsBold in Style;
      FontItalic := fsItalic in Style;
      FontUnderline := fsUnderline in Style;
      FontStrikeOut := fsStrikeOut in Style;
      FontColor := Color;
      FontScript := Charset;
   end;
   if not fdListViewFont.Execute then
      with fdListViewFont.Font do
      begin
         Name := FontName;
         Size := FontSize;
         Style := [];
         if FontBold then
            Style := Style + [fsBold];
         if FontItalic then
            Style := Style + [fsItalic];
         if FontUnderline then
            Style := Style + [fsUnderline];
         if FontStrikeOut then
            Style := Style + [fsStrikeOut];
         Color := FontColor;
         Charset := FontScript;
      end;
end;

procedure TfrmOptions.cbAutomaticFontClick(Sender: TObject);
begin
   btnChooseFont.Enabled := not cbAutomaticFont.Checked;
end;

procedure TfrmOptions.edtVBExePathWindowProc(var Msg: TMessage);
var
   Buffer: array[0..MAX_PATH] of WideChar;
   wstrTemp: WideString;
begin
   if Msg.Msg = WM_DROPFILES then
   begin
      Application.BringToFront;
      frmMain.Repaint;
      Repaint;
      case DragQueryFileW(msg.WParam, $FFFFFFFF, nil, 0) of
         1:
            begin
               DragQueryFileW(msg.WParam, 0, @Buffer, sizeof(Buffer));
               DragFinish(Msg.WParam);
               wstrTemp := WideString(Buffer);
               if WideFileExists(wstrTemp) then
               begin
                  odSearchExe.FileName := wstrTemp;
                  edtVBExePath.Text := wstrTemp;
                  edtVBExePath.SetFocus;
                  edtVBExePath.SelStart := Length(edtVBExePath.Text);
                  edtVBExePath.SelLength := 0;
               end
               else
               begin
                  msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
                  PostMessage(Handle, WM_USER + 1024, 0, 0);
                  MessageBoxW(Handle,
                     PWideChar(GetLangTextDef(idxOptions, ['Messages', 'NotAFile'], 'This is not a file !')),
                     PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
               end;
            end;
      else
         DragFinish(msg.WParam);
         msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
         PostMessage(Handle, WM_USER + 1024, 0, 0);
         MessageBoxW(Handle,
            PWideChar(GetLangTextDef(idxOptions, ['Messages', 'JustOneItem'], 'Just one item at a time !')),
            PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      end;
   end
   else
      originaledtVBExePathWindowProc(Msg);
end;

procedure TfrmOptions.edtqExePathWindowProc(var Msg: TMessage);
var
   Buffer: array[0..MAX_PATH] of WideChar;
   wstrTemp: WideString;
begin
   if Msg.Msg = WM_DROPFILES then
   begin
      Application.BringToFront;
      frmMain.Repaint;
      Repaint;
      case DragQueryFileW(msg.WParam, $FFFFFFFF, nil, 0) of
         1:
            begin
               DragQueryFileW(msg.WParam, 0, @Buffer, sizeof(Buffer));
               DragFinish(Msg.WParam);
               wstrTemp := WideString(Buffer);
               if WideFileExists(wstrTemp) then
               begin
                  edtQExePath.Text := wstrTemp;
                  edtQExePath.SetFocus;
                  edtQExePath.SelStart := Length(edtqExePath.Text);
                  edtQExePath.SelLength := 0;
               end
               else
               begin
                  msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
                  PostMessage(Handle, WM_USER + 1024, 0, 0);
                  MessageBoxW(Handle,
                     PWideChar(GetLangTextDef(idxOptions, ['Messages', 'NotAFile'], 'This is not a file !')),
                     PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
               end;
            end;
      else
         DragFinish(msg.WParam);
         msgBoxCaption := GetLangTextDef(idxMessagesType, ['Warning'], 'Warning');
         PostMessage(Handle, WM_USER + 1024, 0, 0);
         MessageBoxW(Handle,
            PWideChar(GetLangTextDef(idxOptions, ['Messages', 'JustOneItem'], 'Just one item at a time !')),
            PWideChar(msgBoxCaption), MB_OK or MB_ICONWARNING);
      end;
   end
   else
      originaledtQExePathWindowProc(Msg);
end;

procedure TfrmOptions.sbVirtualBoxClick(Sender: TObject);
begin
   sbVirtualBox.Down := True;
   if not sbQEMU.Down then
      Exit;
   sbQEMU.Down := False;
end;

procedure TfrmOptions.sbQEMUClick(Sender: TObject);
begin
   sbQEMU.Down := True;
   if not sbVirtualBox.Down then
      Exit;
   sbVirtualBox.Down := False;
end;

end.

